/*
 * Geometric operations.
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __GEO_H__
#define __GEO_H__

#include <math.h>
#include "bool.h"

typedef union
{
    struct { double x, y, z; };
    double axis[3];
} vectorf;

double within(double value, double mag);
bool vectorf_null(const vectorf *in);
void vectorf_reset(vectorf *in);
void vectorf_cross(const vectorf *in1, const vectorf *in2, vectorf *out);
double vectorf_dot(const vectorf *in1, const vectorf *in2);
double vectorf_mag(const vectorf *in);
void vectorf_normalise(vectorf *in, vectorf *out);
void plane_rotate(vectorf *in1, vectorf *in2, double angle);
void vectorf_negate(vectorf *in, vectorf *out);
double vectorf_direction(vectorf *in);
double vectorf_elevation(vectorf *in);
void vectorf_polar(double direction, double elevation, vectorf *out);
void vectorf_convert(const vectorf *in, const vectorf *devicex, const vectorf *devicey, const vectorf *devicez, vectorf *out);
void vectorf_convert_into(const vectorf *in, const vectorf *x, const vectorf *y, const vectorf *z, vectorf *out);
double vectorf_tilt_in_plane(const vectorf *in, const vectorf *x, const vectorf *y);
double vectorf_tilt_in_plane_with_axis(const vectorf *in, const vectorf *x, const vectorf *y, const vectorf *z);
void vectorf_rotate_in_space(const vectorf *in, const vectorf *x, const vectorf *y, const vectorf *z, double angle, vectorf *out);

#define degrad(x)                   ((x) * M_PI / 180.0)
#define raddeg(x)                   ((x) * 180.0 / M_PI)

#define fdiv(x, y)                  (floor((int) ((x) / (y))))

/* Get the closest number at the given resolution. */

#define closest(x, res)             (((int) (x) / (res)) * (res))

/* Get the maximum and minimum of the given numbers. */

#define max(x, y)                   ((x) >= (y) ? (x) : (y))
#define min(x, y)                   ((x) <= (y) ? (x) : (y))

/* Get the sign of a number. */

#define sign(x)                     ((x) == 0 ? 0 : (x) < 0 ? -1 : 1)

/* Eliminate noise using the given threshold. */

#define noise(x, y)                 (fabs(x) < (y) ? 0 : (x))

#endif /* __GEO_H__ */
