/*
 * Ben NanoNote graphical user interface utilities.
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __GUI_H__
#define __GUI_H__

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "ui.h"
#include "geo.h"

typedef struct
{
    uint16_t    x, y;
    uint8_t     r, g, b, a;
} gui_printer;

/* Device properties. */

#define SCREEN_WIDTH            320
#define SCREEN_HEIGHT           240

#define SCREEN_COLUMN_WIDTH     8
#define SCREEN_ROW_HEIGHT       8

#define GUI_BUFSIZE             256

#define PROJECTION_FACTOR       320
#define SKY_GRID_STEP           15

/* Colours in RGBA format. */

#define SKY_UPPER_COLOUR        0xff0000ff
#define SKY_LOWER_COLOUR        0x0000ffff
#define MOTION_REVERSE_COLOUR   0xff00ffff
#define MOTION_FORWARD_COLOUR   0x00ff00ff

/* Access functions. */

void gui_init();
void gui_display_init();
void gui_shutdown(int signum);
void gui_shutdown_threaded(int signum);
void gui_quit();
void gui_clear();
void gui_fill(uint8_t r, uint8_t g, uint8_t b);
void gui_next_row(gui_printer *printer, uint16_t rows);
void gui_next_column(gui_printer *printer, uint16_t columns);
int gui_printf(const char *format, ...);
void gui_sky(vectorf *viewx, vectorf *viewy, vectorf *viewz);
void gui_sky_vertical(vectorf *viewx, vectorf *viewy, vectorf *viewz, int direction, int elevation, int16_t x, int16_t y, uint32_t colour);
void gui_motion(vectorf *viewx, vectorf *viewy, vectorf *viewz, vectorf *accelerationD);
void gui_point(vectorf *viewx, vectorf *viewy, vectorf *viewz, vectorf *point, uint8_t r, uint8_t g, uint8_t b, uint8_t a);
void gui_origin(vectorf *viewx, vectorf *viewy, vectorf *viewz, vectorf *point);
void gui_bar(vectorf *value);
void gui_plot(uint16_t x, uint16_t y, uint8_t r, uint8_t g, uint8_t b, uint8_t a);
void gui_flush();
imu_ui_op gui_handle_events();

#endif /* __GUI_H__ */
