/*
 * Ben NanoNote communication with the Pololu MinIMU-9 with the L3G4200D 3-axis
 * gyroscope and LSM303DLM accelerometer/magnetometer.
 *
 * http://www.pololu.com/catalog/product/1265
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __MEASURE_H__
#define __MEASURE_H__

#include "geo.h"

/* Function definitions. */

void ui_calibrate(bool using_filter, int (*print)(const char *, ...), void (*flush)());
void mag_calibrate();
void *get_measurements(void *arg);

#ifdef __MEASURE_H_PRIVATE__

/* The raw accelerometer value of 1g. */

const double acceleration1g = 1000000.0 / IMU_UG_FACTOR;

/* The device vectors defined in the chosen coordinate system, together with
   the corresponding initial view axes. */

const vectorf devicex0 = {{0, 0, -1}}, viewz0 = {{0, 0, 1}},
              devicey0 = {{-1, 0, 0}}, viewx0 = {{1, 0, 0}},
              devicez0 = {{0, -1, 0}}, viewy0 = {{0, 1, 0}};

/* Measurements. */

vectorf accelerationB[IMU_ACCEL_BUFFER_SIZE],
        acceleration,
        acceleration0 = {{0, 0, 0}},
        rotation,
        rotation0 = {{0, 0, 0}},
        field,
        fieldmin = {{-319.0, -303.0, -333.0}},
        fieldmax = {{101.0, 124.0, 87.0}};

/* Timekeeping. */

struct timeval imu_updated, imu_magnet_updated;
uint32_t imu_period = 0;

/* Orientation. */

vectorf devicex, devicey, devicez,
        viewx, viewy, viewz,
        accelerationD,
        accelerationR = {{0, 0, -1}},
        accelerationRD = {{0, 1, 0}},
        fieldD = {{0, 0, 0}},
        fieldD0 = {{0, 0, 0}},
        field0 = {{0, 0, 0}},
        fieldN = {{0, 0, 0}},
        fieldE = {{0, 0, 0}};
double direction, elevation, tilt,
       elevationA = 0, tiltA = 0,
       directionF, elevationF,
       directionF0, elevationF0;

/* Measurement thread. */

pthread_t thread;
pthread_mutex_t mutex;

#else

extern const double acceleration1g;
extern vectorf devicex0, devicey0, devicez0,
               devicex, devicey, devicez,
               viewx0, viewy0, viewz0,
               viewx, viewy, viewz,
               accelerationB[IMU_ACCEL_BUFFER_SIZE],
               acceleration,
               acceleration0,
               accelerationD,
               accelerationR,
               accelerationRD,
               rotation,
               rotation0,
               field,
               fieldmin, fieldmax,
               fieldD, fieldD0, field0, fieldN, fieldE;
extern struct timeval imu_updated;
extern uint32_t imu_period;
extern double direction, elevation, tilt,
              elevationA, tiltA,
              directionF, elevationF,
              directionF0, elevationF0;

/* Measurement thread. */

extern pthread_t thread;
extern pthread_mutex_t mutex;

#endif /* __MEASURE_H_PRIVATE__ */

#endif /* __MEASURE_H__ */
